package com.bocloud.sms.booter.controller;

import com.auth0.jwt.interfaces.Claim;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.interfaces.UserService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.Tokens;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * @author tyl
 * @since 2020/7/24 周五 11:38
 */
@RestController
@Tag(name = "token管理")
public class TokenController {

    private final StringRedisTemplate redisTemplate;
    private final TenantService tenantService;
    private final UserService userService;

    public TokenController(StringRedisTemplate redisTemplate, TenantService tenantService, UserService userService) {
        this.redisTemplate = redisTemplate;
        this.tenantService = tenantService;
        this.userService = userService;
    }

    /**
     * 单点登录验证token
     *
     * @param request 请求
     * @return token
     */
    @GetMapping("/v1/token")
    @Operation(summary = "单点登录验证token")
    public GeneralResult<String> checkToken(HttpServletRequest request) {
        String token = request.getHeader(Common.TOKEN);
        if (!StringUtils.hasText(token)) {
            token = request.getParameter(Common.TOKEN);
        }
        String tokenKey = "";
        if (StringUtils.hasText(token)) {
            Claim claim = Tokens.parse(token);
            if (claim == null) {
                return new GeneralResult<>(false, "token校验失败");
            }
            String catalog = claim.asMap().get("catalog").toString();
            long uuid = Long.parseLong(claim.asMap().get(Common.UUID).toString());
            tokenKey = Common.TOKEN + "_" + catalog + "_" + uuid;
        }
        //校验token
        boolean checkResult = check(token, tokenKey);
        if (checkResult) {
            String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
            return new GeneralResult<>(true, newTokenCache, "校验token成功");
        } else {
            return new GeneralResult<>(false, "token校验失败");
        }
    }

    private boolean check(String token, String tokenKey) {
        String newTokenCache = (String) this.redisTemplate.opsForHash().get(tokenKey + "_n", Common.TOKEN);
        if (token.equals(newTokenCache)) {
            return true;
        } else {
            String oldTokenCache = this.redisTemplate.opsForValue().get(tokenKey + "_o");
            return token.equals(oldTokenCache);
        }
    }

    @PostMapping("/v1/sso/token/info")
    @Operation(summary = "token解析获取信息")
    public GeneralResult<Map<String, Object>> getTokenInfo(HttpServletRequest request) {
        String token = request.getHeader(Common.TOKEN);
        HttpSession session = request.getSession();
        if (!StringUtils.hasText(token)) {
            return new GeneralResult<>(false, "token不存在");
        }
        Claim claim = Tokens.parse(token);
        if (claim == null) {
            return new GeneralResult<>(false, "token信息异常重新登录");
        }
        String catalog = claim.asMap().get("catalog").toString();
        String account = claim.asMap().get(Common.ACCOUNT).toString();
        GeneralResult<Map<String, Object>> result;
        if (catalog.equals(RequestContext.Catalog.Tenant.name())) {
            result = tenantService.tokenLogin(account);
        } else {
            result = userService.tokenLogin(account);
        }
        Map<String, Object> object = result.getData();
        object.put(Common.TOKEN, token);
        object.put("redirect", session.getAttribute("redirect"));
        result.setData(object);
        return result;
    }
}
